"""
This module contains functionality for accessing System Diagnostics

Do NOT import this module directly. The L{PLConnection
<packetlogic2.PLConnection>} object should be used, e.g:

    >>> import packetlogic2
    >>> pl = packetlogic2.connect("192.168.1.25", "admin", "pldemo00")
    >>> ht = pl.SysDiag()

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

from resource import Resource
from thisversion import PLDB

import plutil
import sys

_Prop_SysDiagZone = plutil.Prop ('oid', 'name', 'num_values', valuedefs=dict)

class SysDiagZone(object):
    """
    Information about a System Diagnostiscs Zone

    Returned in a list by L{SysDiag.zone_list}.
    """

    TYPE_MOMENT = PLDB.SYSDIAG_ZONE_TYPE_MOMENT
    TYPE_SIZE = PLDB.SYSDIAG_ZONE_TYPE_SIZE
    TYPE_SPEED = PLDB.SYSDIAG_ZONE_TYPE_SPEED
    TYPE_TIMESPAN = PLDB.SYSDIAG_ZONE_TYPE_TIMESPAN
    TYPE_PERCENT = PLDB.SYSDIAG_ZONE_TYPE_PERCENT
    TYPE_TIME = PLDB.SYSDIAG_ZONE_TYPE_TIME
    TYPE_BITMASK = PLDB.SYSDIAG_ZONE_TYPE_BITMASK
    TYPE_MEMORY = PLDB.SYSDIAG_ZONE_TYPE_MEMORY

    _valid_value_types = (
        TYPE_MOMENT,
        TYPE_SIZE,
        TYPE_SPEED,
        TYPE_TIMESPAN,
        TYPE_PERCENT,
        TYPE_TIME,
        TYPE_BITMASK,
        TYPE_MEMORY,
        )

    FLAGS_DEBUG_ONLY = PLDB.SYSDIAG_ZONE_FLAGS_DEBUG_ONLY
    FLAGS_SINGLE_INSTANCE = PLDB.SYSDIAG_ZONE_FLAGS_SINGLE_INSTANCE
    FLAGS_HIDDEN = PLDB.SYSDIAG_ZONE_FLAGS_HIDDEN


    _Prop_Alert = plutil.Prop ('minimum', 'minimum_rate', 'maximum', 'maximum_rate', comment=str, trigger=str)

    class Alert(object):
        """
        Information about a System Diagnostiscs Alert Entry
        """

        __slots__ = ('_prop', '_oid', '_vd')

        def __init__ (self, _oid=None, _vd=None, **props):
            """
            Should not be instantiated manually.
            """
            self._oid = _oid
            self._vd = _vd
            self._prop = SysDiagZone._Prop_Alert (**props)
        def __repr__(self):
            return "<%s %s %d %d %d %d>" % (self.__class__.__name__, self.oid, self.minimum, self.minimum_rate, self.maximum, self.maximum_rate)

        def update(self, **props):
            self._prop.update (**props)
            pldb = self._vd._zone._pldb
            plutil._do(pldb, pldb.session.sysdiag_level_update (self.oid,
                                                                self.minimum,
                                                                self.maximum,
                                                                self.minimum_rate,
                                                                self.maximum_rate,
                                                                self.comment,
                                                                self.trigger),
                       plutil._AddCB(pldb))

        oid = property(lambda self: self._vd._zone.oid + "." + str(self._vd.id), doc="Value OID")
        minimum = property(lambda self: self._prop.minimum, doc="Minimum value required to trigger the alert")
        minimum_rate = property(lambda self: self._prop.minimum_rate, doc="Minimum rate required to trigger the alert")
        maximum = property(lambda self: self._prop.maximum, doc="Maximum value required to trigger the alert")
        maximum_rate = property(lambda self: self._prop.maximum_rate, doc="Maximum rate required to trigger the alert")
        comment = property(lambda self: self._prop.comment, doc="Alert comment")
        trigger = property(lambda self: self._prop.trigger, doc="Trigger script to run when one of the alert criteria is met")
    
    _Prop_Value = plutil.Prop (id=int, level=list, value=int, rate=int, min=int, max=int, min_tstamp=int, max_tstamp=int, alert_flags=int, systemid=str)

    class Value(object):
        """
        Information about a System Diagnostiscs Value
        """
        __slots__ = ('_prop', '_definition')

        def __init__ (self, _definition=None, **props):
            """
            Should not be instantiated manually, use L{zone_list <SysDiag.zone_list>}.
            """
            self._definition = _definition
            self._prop = SysDiagZone._Prop_Value (**props)

        def __repr__(self):
            return "<%s #%u '%s' value=%d rate=%d @%s>" % (self.__class__.__name__, self._prop.id, "/".join(self._prop.level), self._prop.value, self._prop.rate, self._prop.systemid)

        def __int__(self):
            return self._prop.value

        def __iadd__(self, v):
            self._prop.value += int (v)
            return self._prop.value

        def __isub__(self, v):
            self._prop.value -= int (v)
            return self._prop.value
        
        definition = property(lambda self: self._definition, doc="Value definition")
        id = property(lambda self: self._prop.id, doc="Value ID")
        level = property(lambda self: self._prop.level, doc="Value level")
        systemid = property(lambda self: self._prop.systemid, doc="Value systemid if any")
        value = property(lambda self: self._prop.value, doc="Value")
        rate = property(lambda self: self._prop.rate, doc="Value rate")
        min = property(lambda self: self._prop.min, doc="Minimum value seen")
        max = property(lambda self: self._prop.max, doc="Maximum value seen")
        min_tstamp = property(lambda self: self._prop.min_tstamp, doc="Timestamp of min")
        max_tstamp = property(lambda self: self._prop.max_tstamp, doc="Timestamp of max")
        alert_flags = property(lambda self: self._prop.alert_flags, doc="Alert flags")

    _Prop_ValueDefinition = plutil.Prop ('id', 'name', 'type', 'flags')

    class ValueDefinition(object):
        """
        Information about a System Diagnostiscs Value
        """
        __slots__ = ('_prop', '_zone', '_value', '_alert')

        def __init__ (self, _zone=None, **props):
            """
            Should not be instantiated manually, use L{zone_list <SysDiag.zone_list>}.
            """
            self._zone = _zone
            self._prop = SysDiagZone._Prop_ValueDefinition (**props)
            level = []
            level.append (self.name)
            self._value = SysDiagZone.Value (_definition=self, id=self.id, level=level, value=-sys.maxint-1, rate=-sys.maxint-1, min=-sys.maxint-1, max=-sys.maxint-1)
            self._alert = SysDiagZone.Alert (_vd=self, minimum=-1, maximum=-1, minimum_rate=-1,maximum_rate=-1)

        def __repr__(self):
            return "<%s #%u %s>" % (self.__class__.__name__, self._prop.id, repr(self._prop.name))

        def __int__(self):
            return self._value.value

        def __iadd__(self, v):
            self._value._prop.value += int (v)
            return self._value._prop.value

        def __isub__(self, v):
            self._value._prop.value -= int (v)
            return self._value._prop.value

        def values(self):
            """
            Lists all sub-levels for this value
            
            @rtype: C{list} of L{SysDiagZone.Value}
            @returns: List containing L{SysDiagZone.Value} objects.
            """
            return map (self._set_definition,
                        plutil._do(self._zone._pldb, self._zone._pldb.session.sysdiag_zone_value_get (self._zone.oid, self.id),
                                   plutil._ListCB (self, self._zone._pldb.session.sysdiag_zone_value_get_handle, SysDiagZone.Value, False)))

        def _set_definition(self, value):
            value._definition = self
            return value


        id = property(lambda self: self._prop.id, doc="Value ID")
        name = property(lambda self: self._prop.name, doc="Name of the value")
        type = property(lambda self: self._prop.type, doc="Value type")
        flags = property(lambda self: self._prop.flags, doc="Value flags")
        value = property(lambda self: self._value, doc="Top-level value")
        alert = property(lambda self: self._alert, doc="Value alert levels")

    __slots__ = ('_prop', '_pldb')

    def __init__ (self, _pldb=None, **props):
        """
        Should not be instantiated manually, use L{zone_list <SysDiag.zone_list>}.
        """
        self._pldb = _pldb
        self._prop = _Prop_SysDiagZone (**props)
        for id in self._prop.valuedefs:
            self._prop.valuedefs[id]._zone = self

    def __repr__(self):
        return "<%s '%s' '%s'>" % (self.__class__.__name__, self._prop.oid, self._prop.name)

    def __getitem__(self, key):
        if isinstance (key, int):
            return self._prop.valuedefs[key]
        elif isinstance (key, basestring):
            for (i, vd) in self._prop.valuedefs.iteritems():
                if vd.name == key:
                    return vd
            raise IndexError
        else:
            raise TypeError

    def __setitem__(self, key, value):
        vd = self[key]
        vd.value._prop.value = int (value)

    def refresh(self):
        """
        Refreshes all top-level values for this zone
        """
        values = plutil._do(self._pldb, self._pldb.session.sysdiag_zone_get (self.oid),
                            plutil._ListCB (self, self._pldb.session.sysdiag_zone_get_handle, SysDiagZone.Value, False))
        for v in values:
            if v.id in self._prop.valuedefs:
                # PLDB doesn't provide any names from pldb_api_sysdiag_zone_get()
                # fill in the name here for good measure.
                v._prop.level = []
                v._prop.level.append (self.valuedefs[v.id].name)
                self._prop.valuedefs[v.id]._value = v

    def value_definition_add(self, id, name, type, flags = 0):
        """
        Adds a new value definition to the zone, this should be done before
        registering the zone with PLDB
        """

        if type not in self._valid_value_types:
            raise TypeError("Invalid value type")
        vd = SysDiagZone.ValueDefinition (_zone = self, id = id, name = name, type = type, flags = flags)
        self._prop.valuedefs[id] = vd
        self._prop.num_values += 1

    def register(self):
        """
        Register a new zone with PLDB
        """
        plutil._do(self._pldb, self._pldb.session.sysdiag_zone_register (self), plutil._AddCB(self._pldb))

    def send(self, elapsed):
        """
        Update zone values

        @type elapsed: C{int}
        @param elapsed: Time in ms since last update

        """

        values = { }
        for id in self._prop.valuedefs:
            if int(self[id]) != -sys.maxint-1:
                values[id] = self[id].value.value
        
        plutil._do(self._pldb, self._pldb.session.sysdiag_zone_update_values (self.oid, elapsed, values),
                   plutil._AddCB(self._pldb))

    def _setzone(self, o):
        o._zone = self
        return o

    oid = property(lambda self: self._prop.oid, doc="Zone OID")
    name = property(lambda self: self._prop.name, doc="Name of the zone")
    num_values = property(lambda self: self._prop.num_values, doc="Number of value definitions in the zone")
    valuedefs = property(lambda self: self._prop.valuedefs, doc="Dict of value definitions")

_Prop_SysDiagLogEntry = plutil.Prop ('id', 'tstamp', 'is_new', 'severity', 'title', 'body')

class SysDiagLogEntry(object):
    """
    Information about a System Diagnostiscs Log Entry

    Returned in a list by L{SysDiag.log_list}.
    """

    __slots__ = ('_prop', '_pldb')

    def __init__ (self, _pldb=None, **props):
        """
        Should not be instantiated manually, use L{log_list <SysDiag.log_list>}.
        """
        self._pldb = _pldb
        self._prop = _Prop_SysDiagLogEntry (**props)

    def ack(self):
        """
        Clears this entry's is_new flag.
        """
        plutil._do(self._pldb, self._pldb.session.sysdiag_log_ack (self._prop.id), plutil._AddCB(self._pldb))
        self._prop.is_new = False

    id = property(lambda self: self._prop.id, doc="ID number of the event")
    tstamp = property(lambda self: self._prop.tstamp, doc="Timestamp of the event")
    is_new = property(lambda self: self._prop.is_new, doc="Status of the event")
    severity = property(lambda self: self._prop.severity, doc="Event severity")
    title = property(lambda self: self._prop.title, doc="Title of the event")
    body = property(lambda self: self._prop.body, doc="Event details")
    
_Prop_SysDiagMallocUsageEntry = plutil.Prop ('systemname', 'zonename', names=list)
    
class SysDiagMallocUsageEntry(object):
    """
    Information about a System Diagnostiscs Malloc Usage Entry

    Returned in a list by L{SysDiag.log_list}.
    """
    class _FetchCB(plutil._Callback):
	def __init__(self, pldb):
            plutil._Callback.__init__(self, pldb)
            self.names = [ ]
            self._pldb = pldb

	def handleOK(self):
            return self.names

	def handleData(self, msg):
		self.names = self._pldb.session.sysdiag_malloc_usage_get_handle (msg, SysDiagMallocUsageEntry.Name)

    _Prop_Name = plutil.Prop ('name', 'bytes', 'total_allocations', 'current_allocations')
    class Name(object):
        __slots__ = ('_prop')

        def __init__ (self, **props):
            """
            Should not be instantiated manually, use L{log_list <SysDiag.log_list>}.
            """
            self._prop = SysDiagMallocUsageEntry._Prop_Name (**props)

        def __repr__(self):
            return "<%s '%s' %u>" % (self.__class__.__name__, self._prop.name, self._prop.bytes)

        name = property(lambda self: self._prop.name, doc="Name of the value")
        bytes = property(lambda self: self._prop.bytes, doc="Current allocation size in bytes")
        total_allocations = property(lambda self: self._prop.total_allocations, doc="Total number of allocations")
        current_allocations = property(lambda self: self._prop.current_allocations, doc="Current number of allocations")

    __slots__ = ('_prop', '_pldb')

    def __init__ (self, _pldb=None, **props):
        """
        Should not be instantiated manually, use L{log_list <SysDiag.log_list>}.
        """
        self._pldb = _pldb
        self._prop = _Prop_SysDiagMallocUsageEntry (**props)

    def __repr__(self):
        return "<%s '%s' '%s'>" % (self.__class__.__name__, self._prop.systemname, self._prop.zonename)

    def fetch(self):
        """
        Clears this entry's is_new flag.
        """
        self._prop.names = plutil._do(self._pldb, self._pldb.session.sysdiag_malloc_usage_get (self._prop.systemname, self._prop.zonename),
                                      self._FetchCB(self._pldb))

    systemname = property(lambda self: self._prop.systemname, doc="Name of the system")
    zonename = property(lambda self: self._prop.zonename, doc="Name of the zone")
    names = property(lambda self: self._prop.names, doc="List of malloc names")

class SysDiag(Resource):
    LOG_SEVERITY_INFO = PLDB.SYSDIAG_LOG_SEVERITY_INFO
    LOG_SEVERITY_WARNING = PLDB.SYSDIAG_LOG_SEVERITY_WARNING
    LOG_SEVERITY_CRITICAL = PLDB.SYSDIAG_LOG_SEVERITY_CRITICAL

    def _resource(self):
        return PLDB.RESOURCE_SYSDIAG

    def zone_version(self):
        return plutil._do(self, self.session.sysdiag_zone_version(), plutil._AddCB(self, self.session.sysdiag_zone_version_handle))
    
    def zone_list(self):
        """
        Lists all available system diagnostics zones

        @rtype: C{dict} of L{SysDiagZone}
        @returns: Dict containing L{SysDiagZone} objects with OID as the key.
        """
        
        zones = plutil._do(self, self.session.sysdiag_zone_list (),
                          plutil._ListCB (self, self.session.sysdiag_zone_list_handle, SysDiagZone))
        ret = { }
        for zone in zones:
            ret[zone.oid] = zone

        alerts = plutil._do(self, self.session.sysdiag_level_list (),
                          plutil._ListCB (self, self.session.sysdiag_level_list_handle, SysDiagZone.Alert, False))
        for a in alerts:
            lastdot = a._oid.rfind ('.')
            if lastdot == -1:
                continue
            zone = a._oid[0:lastdot]
            index = int (a._oid[lastdot + 1:])
            if zone in ret:
                if index in ret[zone].valuedefs:
                    a._vd = ret[zone][index]
                    ret[zone][index]._alert = a
        return ret

    def zone_create(self, oid, name):
        return SysDiagZone(_pldb = self, oid = oid, name = name, num_values = 0)

    def zone_system_name_set(self, systemid, systemname):
        """
        Sets the systemid and system-name used for updating zone values
        Must be called before using SysDiagZone.update_values()
        """
        plutil._do(self, self.session.sysdiag_zone_system_name_set (systemid, systemname), plutil._AddCB(self))
        
    def log_post(self, severity, title, body):
        """
        Post a new entry to the System Diagnostics log.

        @type severity: C{int}
        @param severity: Event severity. (SYSDIAG_LOG_SEVERITY_INFO, SYSDIAG_LOG_SEVERITY_WARNING or SYSDIAG_LOG_SEVERITY_CRITICAL)

        @type title: C{str}
        @param title: Event title.

        @type body: C{str}
        @param body: Event body.
        """
        plutil._do (self, self.session.sysdiag_log_post (severity, title, body), plutil._AddCB (self))

    def log_list(self, only_new=False, min_id=0):
        """
        Lists system diagnostics log entries.

        @type only_new: C{bool}
        @param only_new: Only return entries with the is_new flag set.

        @type min_id: C{int}
        @param min_id: Only return entries with a id > min_id

        @rtype: C{list} of L{SysDiagLogEntry}
        @returns: List containing L{SysDiagLogEntry} objects.
        """
        
        return plutil._do(self, self.session.sysdiag_log_list (only_new, min_id),
                          plutil._ListCB (self, self.session.sysdiag_log_list_handle, SysDiagLogEntry))

    def malloc_usage_list(self):
        """
        Lists system diagnostics malloc usage entries.

        @rtype: C{list} of L{SysDiagMallocUsageEntry}
        @returns: List containing L{SysDiagMallocUsageEntry} objects.
        """
        
        return plutil._do(self, self.session.sysdiag_malloc_usage_list (),
                          plutil._ListCB (self, self.session.sysdiag_malloc_usage_list_handle, SysDiagMallocUsageEntry))
