"""
This module contains RewriteObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
RewriteObjects.

    >>> o = r.object_add("/RewriteObjects/Test")
    >>> o.add("dstip=192.168.1.1")
    >>> print o.items
    ['dstip=192.168.1.1']

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plobject
from thisversion import PLDB
import plutil
from plutil import verify_ip

def _verify_mac(mac):
	m = mac.split(":")
	if len(mac) != 17 or len(m) != 6:
		return False

	allow = "0123456789abcdefABCDEF"
	for i in m:
		if len(i) != 2 or i[0] not in allow or i[1] not in allow:
			return False
	
	return True

def _verify_dscp(dscp):
	try:
		dscp = int(dscp)
	except:
		return False

	return (dscp >=0 and dscp <= 63)

def _verify_vlanp(vlanp):
	try:
		vlanp = int(vlanp)
	except:
		return False

	return (vlanp >=0 and vlanp <= 7)

def _verify_poolid(poolid):
	try:
		poolid = int(poolid)
	except:
		return False

	return True


class RewriteObject(plobject.PLObject):
	"""
	RewriteObject Implementation

	Object path: /RewriteObjects

	@cvar Item: RewriteObject Item implementation.
	"""

	__slots__ = ('_prop', '_pldb', '_callback', '_item_added')
	
	base_path = plutil.roConst("/RewriteObjects", "Only the type portion of the path")
	type = plutil.roConst(PLDB.OBJECT_TYPE_REWRITEOBJECT, "Number representing the type of object")

	def _create_item(self, value):
		kv = value.split("=")
		if not len(kv) == 2:
			raise ValueError("Format error on string should be prop=value")

		if kv[0] == 'dstmac':
			field = PLDB.REWRITEOBJECT_FIELD_DESTMAC
			if not _verify_mac(kv[1]):
				raise ValueError("Value doesn't seem like a MAC address")
			
		elif kv[0] == 'dstip':
			field = PLDB.REWRITEOBJECT_FIELD_DESTIP
			if not verify_ip(kv[1]):
				raise ValueError("Value doesn't seem like an IP address")
			
		elif kv[0] == 'srcip':
			field = PLDB.REWRITEOBJECT_FIELD_SRCIP
			if not verify_ip(kv[1]):
				raise ValueError("Value doesn't seem like an IP address")
			
		elif kv[0] == 'dscp':
			field = PLDB.REWRITEOBJECT_FIELD_DSCP
			if not _verify_dscp(kv[1]):
				raise ValueError("Value doesn't seem like a DSCP number")
		elif kv[0] == 'vlanp':
			field = PLDB.REWRITEOBJECT_FIELD_VLANP
			if not _verify_vlanp(kv[1]):
				raise ValueError("Value doesn't seem like a valid 802.1p priority")
		elif kv[0] == 'poolid':
			field = PLDB.REWRITEOBJECT_FIELD_POOLID
			if not _verify_poolid(kv[1]):
				raise ValueError("Value doesn't seem like a valid poolid")
		else:
			raise ValueError("Format error. Should use either dstmac, dstip or dscp as key")

		return self.Item(self, field=field, value=kv[1])

	def add(self, value):
		"""
		Add a RewriteObject Item to this RewriteObject

		@type value: C{str}
		@param value: A string that describes the value of the RewriteObject item
			this should be formatted as I{property=value}

		"""
		plobject.PLObject.add(self, value)

	class Item(plobject.PLObject.Item):
		"""
		RewriteObject Item implementation.
		Should not be instantiated, use L{RewriteObject.add} instead.

		@type field: C{int}
		@type value: C{str}
		"""
		__slots__ = ('_prop', '_parent')
		_Prop = plutil.Prop(field=int, value=str, *plobject._item_props)

		def __str__(self):
			fields = {PLDB.REWRITEOBJECT_FIELD_DESTMAC: 'dstmac',
				  PLDB.REWRITEOBJECT_FIELD_DESTIP: 'dstip',
				  PLDB.REWRITEOBJECT_FIELD_SRCIP: 'srcip',
				  PLDB.REWRITEOBJECT_FIELD_DSCP: 'dscp',
				  PLDB.REWRITEOBJECT_FIELD_VLANP: 'vlanp',
				  PLDB.REWRITEOBJECT_FIELD_POOLID: 'poolid',
				  }

			return fields[self._prop.field] + "=" + self._prop.value

		field = property(lambda self: self._prop.field, doc="Internal number representing the field (dstmac, dstip or dscp)")
		value = property(lambda self: self._prop.value, doc="Value of the flag")
